/*****************************************************
Project : Experimental ILDA player
Date    : 29.09.2009
Author  : Sergey                          

Chip type           : ATmega88
Program type        : Application
Clock frequency     : 16,000000 MHz
Memory model        : Small
External SRAM size  : 0
Data Stack size     : 256
*****************************************************/

#include <mega88.h>

// Standard Input/Output functions
#include <stdio.h>
#include <delay.h>
// SPI functions
#include <spi.h>
#include "sdio.c"//  -  
#include "fat.c"//   FAT
#include "ILDA.c"//      ILD
void enable_TIM1_int(void);
void disable_TIM1_int(void);
#include "stream_play.c"//   

flash char all_files[11]={//  -    
'?','?','?','?','?','?','?','?','I','L','D'};

void enable_TIM1_int(void)//   
{
        TCNT1H=0xF9;//  
        TCNT1L=0xC0;//  ~10
        TIMSK1=0x01;//TIM1_OVF -   
}

void disable_TIM1_int(void)//  
{
        TIMSK1=0x00;//TIM1_OVF
}

// Timer 1 overflow interrupt service routine
interrupt [TIM1_OVF] void timer1_ovf_isr(void)
{
read_fifo();//       FIFO
TCNT1H=0xF9;//0xF9;
TCNT1L=0xC0;// 
}

void main(void)
{
// Declare your local variables here 

unsigned int entry_number=0,i;
struct OPEN_RESULT open_result;//   
struct FRAME_RESULT frame;//     

// Crystal Oscillator division factor: 1
#pragma optsize-
CLKPR=0x80;
CLKPR=0x00;
#ifdef _OPTIMIZE_SIZE_
#pragma optsize+
#endif
 
PORTB=0x04;
DDRB=0b00101100;//0x14;

PORTC=0x00;
DDRC=0x00;

PORTD=0x00;
DDRD=0b01100000;//0x02|0x60;//PD1 | PD5 | PD6

// Timer/Counter 0 initialization
// Clock source: System Clock
// Clock value: 16000,000 kHz
// Mode: Phase correct PWM top=FFh
// OC0A output: Non-Inverted PWM
// OC0B output: Non-Inverted PWM
TCCR0A=0xA1;
TCCR0B=0x01;
TCNT0=0x00;
OCR0A=0x127;
OCR0B=0x127;

// Timer/Counter 1 initialization
// Clock source: System Clock
// Clock value: 16000,000 kHz
// Mode: Normal top=FFFFh
// OC1A output: Discon.
// OC1B output: Discon.
// Noise Canceler: Off
// Input Capture on Falling Edge
// Timer 1 Overflow Interrupt: On
// Input Capture Interrupt: Off
// Compare A Match Interrupt: Off
// Compare B Match Interrupt: Off
TCCR1A=0x00;
TCCR1B=0x01;
TCNT1H=0xF9;
TCNT1L=0xC0;
ICR1H=0x00;
ICR1L=0x00;
OCR1AH=0x00;
OCR1AL=0x00;
OCR1BH=0x00;
OCR1BL=0x00;

// Timer/Counter 2 initialization
// Clock source: System Clock
// Clock value: Timer 2 Stopped
// Mode: Normal top=FFh
// OC2A output: Disconnected
// OC2B output: Disconnected
ASSR=0x00;
TCCR2A=0x00;
TCCR2B=0x00;
TCNT2=0x00;
OCR2A=0x00;
OCR2B=0x00;

// External Interrupt(s) initialization
// INT0: Off
// INT1: Off
// Interrupt on any change on pins PCINT0-7: Off
// Interrupt on any change on pins PCINT8-14: Off
// Interrupt on any change on pins PCINT16-23: Off
EICRA=0x00;
EIMSK=0x00;
PCICR=0x00;

// Timer/Counter 0 Interrupt(s) initialization
TIMSK0=0x00;
// Timer/Counter 1 Interrupt(s) initialization
TIMSK1=0x00;
// Timer/Counter 2 Interrupt(s) initialization
TIMSK2=0x00;

// USART initialization
UCSR0A=0x00;
UCSR0B=0x08;
UCSR0C=0x06;
UBRR0H=0x00;
UBRR0L=0x11;//19;//19200//33;//9600

ACSR=0x80;
ADCSRB=0x00;

// SPI initialization
// SPI Type: Master
// SPI Clock Rate: 2000,000 kHz
// SPI Clock Phase: Cycle Half
// SPI Clock Polarity: Low
// SPI Data Order: MSB First
SPCR=0x53;//(62,5KHz) -   SPI    
SPSR=0x00;
    
delay_ms(100);//          

init_SD();// SPI     MMC/SD
 
SPCR=0x50;//    SPI 
SPSR=0x01;//SPI Clock X2

get_FAT();//  
 
#asm("sei");  
///         ///                            
while(entry_number<512)//play all files on SDC
        {
        open_result=*open_file(all_files,entry_number);//    
        entry_number=open_result.entry_number+1;//get next entry
        if(open_result.file_status)//    
                {
                frame=*load_header();//       
                if(!frame.frame_status)//       ,  
                        {
                        goto next;//    
                        };
                total_frames=frame.total_frames;//      
                total_points=frame.total_points;//   ()   

                play_file();//                     
                
        next:
                };
        };

#asm("cli"); 
/// ///
while (1);// 
}
